;=====================================================
; Tomysoft Fontsystem Include V0.5
; 21.03.2005 by Lebostein
;=====================================================
; Mail: Tomysoft@gmx.de
; Page: http://home.arcor.de/tomysoft/
;=====================================================
; Optimized for PureBasic 3.93
;=====================================================

Global FontMapCount  ;Zhlt die vorhandenen Fonts

Enumeration
  #FHL               ;Ausrichtung horizontal linksbndig (LEFT)
  #FHC               ;Ausrichtung horizontal zentriert (CENTER)
  #FHR               ;Ausrichtung horizontal rechtsbndig (RIGHT)
EndEnumeration

Enumeration
  #FVT               ;Ausrichtung vertikal obenbndig (TOP)
  #FVC               ;Ausrichtung vertikal zentriert (CENTER)
  #FVB               ;Ausrichtung vertikal untenbndig (BOTTOM)
EndEnumeration

Structure FontMap
  back.l             ;Transparente Hintergrundfarbe
  over.l             ;Vernderliche Vordergrundfarbe
  wide.w             ;Globaler Zeichenabstand
  high.w             ;Globaler Zeilenabstand
  numx.w             ;Feldanzahl in x
  numy.w             ;Feldanzahl in y
  lenx.w             ;Feldbreite
  leny.w             ;Feldhhe
  maxi.w[256]        ;Reduzierte Zeichenbreite
  posx.w[256]        ;Position x
  posy.w[256]        ;Position y
  draw.l             ;Adresse des Sprites fr die Ausgabe
  copy.l             ;Adresse des Backup-Sprites
  acol.l             ;Aktuelle Farbe des Ausgabesprites
EndStructure

NewList FontMap.FontMap()

;-----------------------------------------------------
; Font Datei laden [OK]
;-----------------------------------------------------

Procedure FontMapLoad(FileName$)

  ;Sprite laden und prfen

  SpriteID = LoadSprite(#PB_Any, FileName$)
  If SpriteID = 0: ProcedureReturn #PB_Default: EndIf

  ;Datei ffnen und prfen

  Length = Len(FileName$) - Len(GetExtensionPart(FileName$))
  FileID = ReadFile(#PB_Any, Left(FileName$, Length) + "FONT")
  If FileID = 0: FreeSprite(SpriteID): ProcedureReturn #PB_Default: EndIf

  ;Neuen Font erstellen und Daten einlesen

  LastElement(FontMap()): AddElement(FontMap())

  FontMap()\back = ReadLong()
  FontMap()\over = ReadLong()
  FontMap()\wide = ReadWord()
  FontMap()\high = ReadWord()
  FontMap()\numx = ReadWord()
  FontMap()\numy = ReadWord()
  FontMap()\lenx = ReadWord()
  FontMap()\leny = ReadWord()

  For y = 1 To FontMap()\numy
  For x = 1 To FontMap()\numx

  code = ReadWord()
  FontMap()\maxi[code] = ReadWord()
  FontMap()\posx[code] = FontMap()\lenx * (x - 1)
  FontMap()\posy[code] = FontMap()\leny * (y - 1)

  Next x
  Next y

  CloseFile(FileID)

  ;Ausgabe-Sprite initialisieren

  FontMap()\draw = SpriteID
  TransparentSpriteColor(FontMap()\draw, Red(FontMap()\back), Green(FontMap()\back), Blue(FontMap()\back))

  ;Backup-Sprite initialisieren (wenn Vordergrundfarbe aktiv)

  If FontMap()\over <> #PB_Default

  FontMap()\acol = FontMap()\over
  FontMap()\copy = CopySprite(FontMap()\draw, #PB_Any)
  TransparentSpriteColor(FontMap()\copy, Red(FontMap()\over), Green(FontMap()\over), Blue(FontMap()\over))

  EndIf

  ;Listenindex als FontID zurckgeben

  FontMapCount = CountList(FontMap())

  ProcedureReturn ListIndex(FontMap())

EndProcedure

;-----------------------------------------------------
; Lnge des Textes ermitteln [OK]
;-----------------------------------------------------

Procedure FontMapWide(FontID, Text$)

  ;FontID prfen und auswhlen

  If FontID < 0 Or FontID >= FontMapCount: ProcedureReturn 0: EndIf
  SelectElement(FontMap(), FontID)

  ;Breite des Textes berechnen

  Count = Len(Text$) - 1
  For CharIndex = 0 To Count
  ASCII = PeekB(@Text$ + CharIndex) & $FF
  If FontMap()\maxi[ASCII] = 0: Continue: EndIf
  PosiX + FontMap()\maxi[ASCII] + FontMap()\wide
  Next CharIndex

  ;Breite des Textes ausgeben

  ProcedureReturn PosiX - FontMap()\wide

EndProcedure

;-----------------------------------------------------
; Hhe des Textes ermitteln [OK]
;-----------------------------------------------------

Procedure FontMapHigh(FontID, Text$)

  ;FontID prfen und auswhlen

  If FontID < 0 Or FontID >= FontMapCount: ProcedureReturn 0: EndIf
  SelectElement(FontMap(), FontID)

  ;Hhe des Textes berechnen

  Lines = CountString(Text$, #CR$) + 1
  PosiY = Lines * (FontMap()\leny + FontMap()\high)

  ;Hhe des Textes ausgeben

  ProcedureReturn PosiY - FontMap()\high

EndProcedure

;-----------------------------------------------------
; Text ausgeben [OK]
;-----------------------------------------------------

Procedure FontMapOutput(FontID, Text$, PosiX, PosiY, Hori, Vert, RGB)

  ;FontID prfen und auswhlen

  If FontID < 0 Or FontID >= FontMapCount: ProcedureReturn 0: EndIf
  SelectElement(FontMap(), FontID)

  ;Sprite einfrben (wenn Vordergrundfarbe aktiv)

  If FontMap()\over <> #PB_Default And FontMap()\acol <> RGB

  UseBuffer(FontMap()\draw)
  ClearScreen(RGB & $FF, RGB >> 8 & $FF, RGB >> 16 & $FF)
  DisplayTransparentSprite(FontMap()\copy, 0, 0)
  UseBuffer(#PB_Default)

  FontMap()\acol = RGB: EndIf

  ;Font auf Bildschirm ausgeben

  LineCount = CountString(Text$, #CR$) + 1

  Select Vert
  Case #FVT: OutPosiY = PosiY
  Case #FVC: OutPosiY = PosiY - FontMapHigh(FontID, Text$) >> 1
  Case #FVB: OutPosiY = PosiY - FontMapHigh(FontID, Text$)
  EndSelect

  For LineIndex = 1 To LineCount

  If LineCount = 1: Line$ = Text$: Else: Line$ = StringField(Text$, LineIndex, #CR$): EndIf

  Select Hori
  Case #FHL: OutPosiX = PosiX
  Case #FHC: OutPosiX = PosiX - FontMapWide(FontID, Line$) >> 1
  Case #FHR: OutPosiX = PosiX - FontMapWide(FontID, Line$)
  EndSelect

  Count = Len(Line$) - 1
  For CharIndex = 0 To Count
  ASCII = PeekB(@Line$ + CharIndex) & $FF
  If FontMap()\maxi[ASCII] = 0: Continue: EndIf
  ClipSprite(FontMap()\draw, FontMap()\posx[ASCII], FontMap()\posy[ASCII], FontMap()\maxi[ASCII], FontMap()\leny)
  DisplayTransparentSprite(FontMap()\draw, OutPosiX, OutPosiY)
  OutPosiX + FontMap()\maxi[ASCII] + FontMap()\wide
  Next CharIndex

  OutPosiY + FontMap()\high + FontMap()\leny

  Next LineIndex

EndProcedure

;-----------------------------------------------------
; Ende der Include
;-----------------------------------------------------
; ExecutableFormat=Windows
; DisableDebugger
; EOF